<!--
SPDX-FileCopyrightText: 2024 Climate Service Center Germany, Technische Universität Dresden

SPDX-License-Identifier: BSD-3-Clause
-->

# Heat2UrbanImpact

[![CI](https://gitlab.dkrz.de/ch1187/plugins4freva/heat2urbanimpact/badges/main/pipeline.svg)](https://gitlab.dkrz.de/ch1187/plugins4freva/heat2urbanimpact/-/pipelines?page=1&scope=all&ref=main)
[![Code coverage](https://gitlab.dkrz.de/ch1187/plugins4freva/heat2urbanimpact/badges/main/coverage.svg)](https://gitlab.dkrz.de/ch1187/plugins4freva/heat2urbanimpact/-/graphs/main/charts)
[![Latest Release](https://gitlab.dkrz.de/ch1187/plugins4freva/heat2urbanimpact/-/badges/release.svg)](https://gitlab.dkrz.de/ch1187/plugins4freva/heat2urbanimpact)
[![Production](https://img.shields.io/badge/Production-Test-ff33fc.svg)](https://ch1187.gitlab-pages.dkrz.de/plugins4freva/heat2urbanimpact/prod_test/)
[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)
[![Imports: isort](https://img.shields.io/badge/%20imports-isort-%231674b1?style=flat&labelColor=ef8336)](https://pycqa.github.io/isort/)
[![PEP8](https://img.shields.io/badge/code%20style-pep8-orange.svg)](https://www.python.org/dev/peps/pep-0008/)
[![Checked with mypy](http://www.mypy-lang.org/static/mypy_badge.svg)](http://mypy-lang.org/)
[![REUSE status](https://api.reuse.software/badge/gitlab.dkrz.de/ch1187/plugins4freva/heat2urbanimpact)](https://api.reuse.software/info/gitlab.dkrz.de/ch1187/plugins4freva/heat2urbanimpact)

## Description
This plugin extracts the development of the atmospheric temperature stratification during a specified heat/warm period from regional climate model data and tailors the data for the usage in urban impact models. For a selectable region of interest and period of time this plugin returns hourly temperature data at different layers/hights in a directly usable format (format specification). This plugin is based on the NUKLEUS ensemble of 3km horizontal grid.

Note that we are developing prototype here and have to make compromises for the first development stage.

## Input: 
Input parameters for the plugin and help texts:
- **shape_file** (): "Select a geo reference file defining your region of interest, if None is selected (default), the whole region that is defined in the climate data will be taken. Note: You can either select a path on the HPC system, or a web url pointing to the file."
- **region** (selectfield): "Select a pre defined German municipality of interest. This selection has only effect if you don't chose a shape file."
- **split_by** (string): "If your selected geo reference file has multiple geometries and you whish to process each geometry separatly you can choose a split key that splits the geometries into different sub regions. The values for the split key will thereby used to distinguish the sub regions."
- **experiment** (selectfield): "Please select the model simulation of interest: historical or projection (global warming level +2K or +3K)."
- **event** (selectfield): "Please select the exceptionality of the heat/warm period by selecting percentiles of temperature probability distribution. Default is the 95th percentile."
- **length_of_event** (integer): "Please enter the number of days of the heat/warm period you would like to receive as output. An integer between 1 and 14 days would be plausible. Default is 3 days."
- **months_of_event** (string): "Please enter the month(s) when your heat/warm event is supposed to happen for your studies. E.g. enter for single month 'June' or for multiple months 'May:August'. Default is 'July'."
- **impact_model** (selectfield): "The format of the output will be tailored for the selected impact model (PALM or ENVI-met)."

Predefined parameters for the prototype:
- project: nukleus
- product: ceu-3
- models: all
- time_frequency: daily and hourly (issue of bias correction)
- variable: tas, tasmin, tasmax (temperature at different hights are not needed for statistical analysis of heat/warm days, but for the final output)

## Method --> still to be figured out!
### Data basis
It is still unclear if hourly bias corrected data of vertical air temperature profiles will be available. Therefore, we see three options to start the data anaylsis:
* option 0 (optimal) - bias-corrected data available: bias-corrected model data is used directly for picking out events and outputting data
* option 1: bias is (proven) not significant: Procedure as variant 0
* option 2*: bias-corrected data are available but only for daily data (mean, min, max): Difference: bias-corrected maximum - uncorrected maximum --> imprinting of this difference on the hourly values of the day under consideration or other method? Method applicable for all height levels? Method applicable for all variables?
* option 3 (most unfavorable): bias is unknown, is judged to be significant and no bias-corrected data: The raw model data are used to derive the climate change signal (e.g. 2K-World - reference period). The climate change signal is suitably imposed on measurement data. Challenge: Derivation of vertical profiles (only measurements at a height of 2 m are available for temperature).

*most plausible and feasible option at the moment

### "Filter out" heat/warm days and create times series of desired length
We are not looking for a specific heat/warm event or a concrete number of heat or summer days in the data. For the chosen experiment (historical, 2K, or 3K) we would like to know how the temperature stratification typically evolves when it is e.g. 2 days exceptionally hot/warm, or 5 days or 10 days. The exceptionality of the heat/warm period is defined with the input parameter _event_ (percentiles of temperature probability distribution) and the length of the heat period is defined with the input parameter _length_of_event_. To consider also the seasonality of heat/warm periods the user can select different months with the input parameter _months_of_event_.

To provide hourly time series is a must-have. We will select a heat/warm day based on bias corrected daily tmax data and then go on as described under option 2 (Data basis section) to imprint the bias correction to the hourly data and different hight levels.

**We still researching how to define realistic time series of heat events using a statistical data analysis or how to develop an artificial timeseries.**

## Final Plugin Output
As a result we get 9 timeseries (ensemble members) with a frequency of 1hr and a length of 1 to 14 days (selectable via _length_of_event_) of vertical temperature stratification (representative for selected heat/warm period via _event_) for the chosen historical, 2K, or 3K experiment which can be fed directly into an urban climate model (format is selectable via _impact_model_).

## Open issues:
* availability of hourly bias corrected data?
* availability of hourly topography corrected data?
* output options:
    * offer plots of all 9 timeseries (+ reference, observation) to visualize ensemble bandwidth (+ difference to past)
* future wish list: 
    * timeseries of reference data and observation data for comparison of 2K/3K future with the past
    * timeseries of additional variables: thermal stress (tas, wind speed, air humidity, short/long-wave irradiation surface), wind speed and direction (optimal: vertical profile or at least at AGS height geostrophic wind), shortwave (direct, diffuse) radiation at surface, air humidity (optimal: vertical profile), boundary conditon at ground surface (only initialisation): soil temperature and humidity 


Barrier-free RCM data, representative for heat events, for urban impact models.


## Connection and Module Loading

To connect to the server and load the necessary modules for the project, use the following commands:

```bash
ssh <user-id>@levante.dkrz.de
module load clint <project-module>
```
whcih in RegKlim project, `<project-module>` is `regiklim-ces`

## Integrating Heat2UrbanImpact with the Plugin

To integrate Heat2UrbanImpact into your Freva instance, follow these steps:

1. **Clone the Plugin Repository:**

    Navigate to the appropriate directory and clone the repository using the following commands:

    ```bash
    cd <project-dir>
    git clone https://gitlab.dkrz.de/ch1187/plugins4freva/heat2urbanimpact.git
    ```
    in RegiKlim, `<project-dir>` is set to `/work/ch1187/regiklim-work/<username>/regiklim-ces/plugins/`

2. **Install Prerequisites:**

    After cloning the repository, install the required packages:

    ```bash
    cd heat2urbanimpact
    make mamba-dep-install
    conda activate plugin_env
    ```

    or simply the following conda command:

    ```bash
    cd heat2urbanimpact
    conda env create --prefix ./plugin_env -f heat2urbanimpact-env.yml
    conda activate plugin_env
    pip install .
    ```

3. **Configure the Freva Instance:**

    Use the following export command to make the plugin available in your Freva instance:

    ```bash
    export EVALUATION_SYSTEM_PLUGINS=<project-dir>/heat2urbanimpact,wrapper
    ```
    For example, in RegiKlim, `<project-dir>` is set to `/work/ch1187/regiklim-work/<username>/plugins/`. You
    can adjust this path based on the specific project you want to plug it into.

Now, the plugin is ready to be used in your Freva instance.

## Development and contribution

To use this in a development setup, clone the [source code][source code] from
gitlab, start the development server and make your changes:

```bash
git clone https://gitlab.dkrz.de/ch1187/plugins4freva/heat2urbanimpact.git
cd heat2urbanimpact
conda env create --prefix ./plugin_env -f heat2urbanimpact-env.yml
conda activate plugin_env
```

More detailed installation instructions my be found in the [docs][docs].


[source code]: https://gitlab.dkrz.de/ch1187/plugins4freva/heat2urbanimpact


For more information on how to contribute to the plugin as well as important details regarding standars and license management chek the [contributing guide][contributing].


## Technical note

This package has been generated from the template
[Freva Plugin Template](https://gitlab.dkrz.de/ch1187/plugins4freva/freva-plugin-template).

See the template repository for instructions on how to update the skeleton for
this package.


## License information

Copyright © 2024 Climate Service Center Germany, Technische Universität Dresden



Code files in this repository are licensed under the
BSD-3-Clause, if not stated otherwise
in the file.


Documentation files in this repository are licensed under CC-BY-4.0, if not stated otherwise in the file.



Supplementary and configuration files in this repository are licensed
under CC0-1.0, if not stated otherwise
in the file.

Please check the header of the individual files for more detailed
information.



### License management

License management is handled with [``reuse``](https://reuse.readthedocs.io/).
If you have any questions on this, please have a look into the
[contributing guide][contributing] or contact the maintainers of
`heat2urbanimpact`.







# Rest of descriptions

## Input: 
Input parameters for the plugin and help texts:
- **shape_file** (): "Select a geo reference file defining your region of interest, if None is selected (default), the whole region that is defined in the climate data will be taken. Note: You can either select a path on the HPC system, or a web url pointing to the file."
- **region** (selectfield): "Select a pre defined German municipality of interest. This selection has only effect if you don't chose a shape file."
- **split_by** (string): "If your selected geo reference file has multiple geometries and you whish to process each geometry separatly you can choose a split key that splits the geometries into different sub regions. The values for the split key will thereby used to distinguish the sub regions."
- **experiment** (selectfield): "Please select the model simulation of interest: historical or projection (global warming level +2K or +3K)."
- **event** (selectfield): "Please select the exceptionality of the heat/warm period by selecting percentiles of temperature probability distribution. Default is the 95th percentile."
- **length_of_event** (integer): "Please enter the number of days of the heat/warm period you would like to receive as output. An integer between 1 and 14 days would be plausible. Default is 3 days."
- **months_of_event** (string): "Please enter the month(s) when your heat/warm event is supposed to happen for your studies. E.g. enter for single month 'June' or for multiple months 'May:August'. Default is 'July'."
- **impact_model** (selectfield): "The format of the output will be tailored for the selected impact model (PALM or ENVI-met)."

Predefined parameters for the prototype:
- project: nukleus
- product: ceu-3
- models: all
- time_frequency: daily and hourly (issue of bias correction)
- variable: tas, tasmin, tasmax (temperature at different hights are not needed for statistical analysis of heat/warm days, but for the final output)

## Method --> still to be figured out!
### Data basis
It is still unclear if hourly bias corrected data of vertical air temperature profiles will be available. Therefore, we see three options to start the data anaylsis:
* option 0 (optimal) - bias-corrected data available: bias-corrected model data is used directly for picking out events and outputting data
* option 1: bias is (proven) not significant: Procedure as variant 0
* option 2*: bias-corrected data are available but only for daily data (mean, min, max): Difference: bias-corrected maximum - uncorrected maximum --> imprinting of this difference on the hourly values of the day under consideration or other method? Method applicable for all height levels? Method applicable for all variables?
* option 3 (most unfavorable): bias is unknown, is judged to be significant and no bias-corrected data: The raw model data are used to derive the climate change signal (e.g. 2K-World - reference period). The climate change signal is suitably imposed on measurement data. Challenge: Derivation of vertical profiles (only measurements at a height of 2 m are available for temperature).

*most plausible and feasible option at the moment

### "Filter out" heat/warm days and create times series of desired length
We are not looking for a specific heat/warm event or a concrete number of heat or summer days in the data. For the chosen experiment (historical, 2K, or 3K) we would like to know how the temperature stratification typically evolves when it is e.g. 2 days exceptionally hot/warm, or 5 days or 10 days. The exceptionality of the heat/warm period is defined with the input parameter _event_ (percentiles of temperature probability distribution) and the length of the heat period is defined with the input parameter _length_of_event_. To consider also the seasonality of heat/warm periods the user can select different months with the input parameter _months_of_event_.

To provide hourly time series is a must-have. We will select a heat/warm day based on bias corrected daily tmax data and then go on as described under option 2 (Data basis section) to imprint the bias correction to the hourly data and different hight levels.

**We still researching how to define realistic time series of heat events using a statistical data analysis or how to develop an artificial timeseries.**

## Final Plugin Output
As a result we get 9 timeseries (ensemble members) with a frequency of 1hr and a length of 1 to 14 days (selectable via _length_of_event_) of vertical temperature stratification (representative for selected heat/warm period via _event_) for the chosen historical, 2K, or 3K experiment which can be fed directly into an urban climate model (format is selectable via _impact_model_).

## Open issues:
* availability of hourly bias corrected data?
* availability of hourly topography corrected data?
* output options:
    * offer plots of all 9 timeseries (+ reference, observation) to visualize ensemble bandwidth (+ difference to past)
* future wish list: 
    * timeseries of reference data and observation data for comparison of 2K/3K future with the past
    * timeseries of additional variables: thermal stress (tas, wind speed, air humidity, short/long-wave irradiation surface), wind speed and direction (optimal: vertical profile or at least at AGS height geostrophic wind), shortwave (direct, diffuse) radiation at surface, air humidity (optimal: vertical profile), boundary conditon at ground surface (only initialisation): soil temperature and humidity 
